package dev.peerat.backend.repository;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dev.peerat.backend.Configuration;

public class DatabaseRepository extends BaseDatabaseQuery{

	private Connection con;	//refractor chain
	private Configuration config;
	
	private DatabasePuzzleRepository puzzleRepo;
	private DatabaseChapterRepository chapterRepo;
	private DatabaseAdminRepository adminRepo;
	private DatabaseAuthRepository authRepo;
	private DatabaseBadgeRepository badgeRepo;
	private DatabaseCompletionRepository completionRepo;
	private DatabaseGroupRepository groupRepo;
	private DatabaseLeaderboardRepository leaderboardRepo;
	private DatabasePlayerRepository playerRepo;

	public DatabaseRepository(ConnectionManager con, Configuration config) {
		super(con);
		this.config = config;
		
		this.puzzleRepo = new DatabasePuzzleRepository(con, config);
		this.chapterRepo = new DatabaseChapterRepository(con, config, puzzleRepo);
		this.adminRepo = new DatabaseAdminRepository(con, config);
		this.authRepo = new DatabaseAuthRepository(con, config);
		this.badgeRepo = new DatabaseBadgeRepository(con, config);
		this.completionRepo = new DatabaseCompletionRepository(con, config, chapterRepo);
		this.groupRepo = new DatabaseGroupRepository(con, config);
		this.leaderboardRepo = new DatabaseLeaderboardRepository(con, config);
		this.playerRepo = new DatabasePlayerRepository(con, config);
	}
	
	public DatabasePuzzleRepository getPuzzleRepository(){
		return this.puzzleRepo;
	}
	
	public DatabaseChapterRepository getChapterRepository(){
		return this.chapterRepo;
	}
	
	public DatabaseAdminRepository getAdminRepository(){
		return this.adminRepo;
	}
	
	public DatabaseAuthRepository getAuthRepository(){
		return this.authRepo;
	}
	
	public DatabaseBadgeRepository getBadgeRepository(){
		return this.badgeRepo;
	}
	
	public DatabaseCompletionRepository getCompletionRepository(){
		return this.completionRepo;
	}
	
	public DatabaseGroupRepository getGroupRepository(){
		return this.groupRepo;
	}
	
	public DatabaseLeaderboardRepository getLeaderboardRepository(){
		return this.leaderboardRepo;
	}
	
	public DatabasePlayerRepository getPlayerRepository(){
		return this.playerRepo;
	}
	
	
	public void hotfix() throws Exception{
		ensureConnection();
		
		PreparedStatement stmt = con.prepareStatement("SELECT c.*, g.* FROM completions c JOIN puzzles p ON c.fk_puzzle = p.id_puzzle JOIN containsGroups cg ON cg.fk_player = c.fk_player JOIN groups g ON g.id_group = cg.fk_group WHERE g.fk_chapter = 12 AND p.fk_chapter = 12;");
		ResultSet results = stmt.executeQuery();
		List<Completionz> list = new ArrayList<>();
		while(results.next()){
			Completionz complete = new Completionz(results);
			list.add(complete);
		}
		Map<Integer, Map<Integer, Completionz>> map = new HashMap<>();
		for(Completionz comp : list){
			Map<Integer,Completionz> puz = map.get(comp.puzzle);
			if(puz == null){
				puz = new HashMap<>();
				map.put(comp.puzzle, puz);
			}
			Completionz c = puz.get(comp.groups);
			if(c == null){
				puz.put(comp.groups, comp);
			}else{
				if(comp.score >= c.score){
					puz.put(comp.groups, comp);
					System.out.println("remove compl "+c.id);
				}
			}
		}
	}
	
	private static class Completionz{
		
		public int id;
		public int puzzle;
		public int player;
		public int tries;
		public int score;
		public int groups;
		
		public Completionz(ResultSet result) throws Exception{
			id = result.getInt("id_completion");
			puzzle = result.getInt("fk_puzzle");
			player = result.getInt("fk_player");
			tries = result.getInt("tries");
			score = result.getInt("score");
			groups = result.getInt("id_group");
			System.out.println(id);
		}
		
	}

}